// API calls related to authentication
package api

import (
	"../badgateway"
	"../helper"
	proxypkg "../proxy"
	"../senddata"
	"../sendfile"
	"fmt"
	"net/http"
	"net/http/httptest"
)

// Reply from auth backend, e.g. for "download from repo" authorization request
type AuthReply struct {
	// raw reply from auth backend & PreAuthorizeHandler().
	// recorded so we can replay it to client in full
	// if e.g. access is rejected.
	RawReply *httptest.ResponseRecorder

	// decoded auth reply
	Response
}

// for detecting whether archive download is ok via senddata mechanism
type testDownloadOkViaSendArchive struct {
	senddata.Prefix
	authReply *AuthReply
}

func (aok *testDownloadOkViaSendArchive) Inject(w http.ResponseWriter, r *http.Request, sendData string) {
	var param struct{ RepoPath string }
	if err := aok.Unpack(&param, sendData); err != nil {
		helper.Fail500(w, fmt.Errorf("testDownloadOkViaSendArchive: unpack sendData: %v", err))
		return
	}

	// if we ever get to this point - auth handler approved
	// access and thus it is ok to download
	aok.authReply.RepoPath = param.RepoPath
}

// Ask auth backend about whether download is ok for a project.
// Authorization is approved if AuthReply.RepoPath != "" on return
// Raw auth backend response is emitted to AuthReply.RawReply
func (a *API) VerifyDownloadAccess(project, query string, header http.Header) AuthReply {
	authReply := AuthReply{
		RawReply: httptest.NewRecorder(),
	}

	// Request to auth backend to verify whether download is possible.
	// - first option is via asking as `git fetch` would do, but on Rails
	//   side this supports only basic auth, not private token.
	// - that's why we auth backend to authenticate as if it was request to
	//   get repo archive and propagate request query and header.
	// url := project + ".git/info/refs?service=git-upload-pack"
	url := project + "/repository/archive.zip"
	if query != "" {
		url += "?" + query
	}
	reqDownloadAccess, err := http.NewRequest("GET", url, nil)
	if err != nil {
		helper.Fail500(authReply.RawReply, fmt.Errorf("GET git-upload-pack: %v", err))
		return authReply
	}
	for k, v := range header {
		reqDownloadAccess.Header[k] = v
	}

	// Send request to auth backend and detect via aok senddata hook
	// whether access is permitted.
	aok := &testDownloadOkViaSendArchive{"git-archive:", &authReply}
	authProxy := senddata.SendData(
		sendfile.SendFile(proxypkg.NewProxy(
			a.URL,
			a.Version,
			a.Client.Transport.(*badgateway.RoundTripper),
		)),
		aok,
	)
	authProxy.ServeHTTP(authReply.RawReply, reqDownloadAccess)

	return authReply
}
